==> dune-fem-2.4.1.tar.gz: dune-fem-2.4.1/dune/fem/space/lagrange/test/basisfunctiontest.cc <==
/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#include "basisfunctiontest.hh"

// #include <dune/grid/io/file/dgfparser/dgfgridtype.hh>
#include <dune/fem/gridpart/common/gridpart.hh>
#include <dune/fem/space/lagrange.hh>

namespace Dune
{

  namespace Fem
  {

    void LagrangeBasis_Test::run()
    {
      testBasisFunctions();
    }

    void LagrangeBasis_Test::testBasisFunctions()
    {
      typedef GridSelector::GridType GridType;
      static const int dimworld = GridSelector::dimworld;

      GridPtr< GridType > gridPtr( gridFile_ );
      GridType& grid = *gridPtr;

      typedef LeafGridPart< GridType > GridPartType;
      GridPartType gridPart( grid );

      typedef FunctionSpace< double, double, dimworld, 1 > FunctionSpaceType;

      // check polynomial order 1
      typedef LagrangeDiscreteFunctionSpace< FunctionSpaceType, GridPartType, 1 >
        OneSpaceType;
      {
        std :: cout << "Linear Basis Functions" << std :: endl;
        OneSpaceType space( gridPart);
        checkLagrangeBasis( space );
      }

      #ifdef TEST_SECOND_ORDER
      // check polynomial order 2
      typedef LagrangeDiscreteFunctionSpace< FunctionSpaceType, GridPartType, 2 >
        TwoSpaceType;
      {
        std :: cout << "Quadratic Basis Functions" << std :: endl;
        TwoSpaceType space( gridPart );
        checkLagrangeBasis( space );
      }
      #endif

      #ifdef TEST_THIRD_ORDER
      typedef LagrangeDiscreteFunctionSpace< FunctionSpaceType, GridPartType, 3 >
        ThreeSpaceType;
      {
        std :: cout << "Cubic Basis Functions" << std :: endl;
        ThreeSpaceType space( gridPart );
        checkLagrangeBasis( space );
      }
      #endif

    }

    template< class SpaceType >
    void LagrangeBasis_Test :: checkLagrangeBasis( const SpaceType &space )
    {
      typedef typename SpaceType :: BasisFunctionSetType  BasisFunctionSetType;
      typedef typename SpaceType :: LagrangePointSetType LagrangePointSetType;
      typedef typename SpaceType :: DomainType DomainType;
      typedef typename SpaceType :: RangeType RangeType;

      int errors = 0;
      for(const auto& entity : space)
      {
        const BasisFunctionSetType& basisSet = space.basisFunctionSet( entity );
        const int numBasisFct = basisSet.size();

        const LagrangePointSetType& pointSet = space.lagrangePointSet( entity );
        const int numPoints = pointSet.size();

        std::vector< RangeType > phi( numPoints, RangeType( 0 ) );

        for( int i = 0; i < numPoints; ++i )
        {

          const DomainType& x = pointSet.point( i );

          // evaluate all basisFunctions on lagrange point i
          basisSet.evaluateAll( x, phi );

          if( std :: abs( phi[ i ][ 0 ] - 1.0 ) >= 1e-10 )
          {
            std :: cout << "Basis function " << i << " failed at " << x
                        << " (" << phi[ i ][ 0 ] << " != 1)!" << std :: endl;
            ++errors;
          }

          for( int j = 0; j < numBasisFct; ++j )
          {
            if( i == j )
              continue;

            // evaluate on lagrange point
            if( std :: abs( phi[ j ][ 0 ] ) >= 1e-10 )
            {
              std :: cout << "Basis function " << j << " failed at " << x
                          << " (" << phi[ j ][ 0 ] << " != 0)!" << std :: endl;
              ++errors;
            }
          }
        }
      }

      assert( errors == 0 );
    }

  } // namespace Fem

} // namespace Dune

==> dune-fem-2.9.0.dev20220404.tar.gz: dune-fem-2.9.0.dev20220404/dune/fem/space/lagrange/test/basisfunctiontest.cc <==
#include "basisfunctiontest.hh"

// #include <dune/grid/io/file/dgfparser/dgfgridtype.hh>
#include <dune/fem/gridpart/common/gridpart.hh>
#include <dune/fem/space/lagrange.hh>

namespace Dune
{

  namespace Fem
  {

    void LagrangeBasis_Test::run()
    {
      testBasisFunctions();
    }

    void LagrangeBasis_Test::testBasisFunctions()
    {
      typedef GridSelector::GridType GridType;
      static const int dimworld = GridSelector::dimworld;

      GridPtr< GridType > gridPtr( gridFile_ );
      GridType& grid = *gridPtr;

      typedef LeafGridPart< GridType > GridPartType;
      GridPartType gridPart( grid );

      typedef FunctionSpace< double, double, dimworld, 1 > FunctionSpaceType;

      // check polynomial order 1
      typedef LagrangeDiscreteFunctionSpace< FunctionSpaceType, GridPartType, 1 >
        OneSpaceType;
      {
        std :: cout << "Linear Basis Functions" << std :: endl;
        OneSpaceType space( gridPart);
        checkLagrangeBasis( space );
      }

      #ifdef TEST_SECOND_ORDER
      // check polynomial order 2
      typedef LagrangeDiscreteFunctionSpace< FunctionSpaceType, GridPartType, 2 >
        TwoSpaceType;
      {
        std :: cout << "Quadratic Basis Functions" << std :: endl;
        TwoSpaceType space( gridPart );
        checkLagrangeBasis( space );
      }
      #endif

      #ifdef TEST_THIRD_ORDER
      typedef LagrangeDiscreteFunctionSpace< FunctionSpaceType, GridPartType, 3 >
        ThreeSpaceType;
      {
        std :: cout << "Cubic Basis Functions" << std :: endl;
        ThreeSpaceType space( gridPart );
        checkLagrangeBasis( space );
      }
      #endif

    }

    template< class SpaceType >
    void LagrangeBasis_Test :: checkLagrangeBasis( const SpaceType &space )
    {
      typedef typename SpaceType :: BasisFunctionSetType  BasisFunctionSetType;
      typedef typename SpaceType :: DomainType            DomainType;
      typedef typename SpaceType :: RangeType             RangeType;
      typedef typename SpaceType :: GridPartType          GridPartType;

      typedef LagrangePointSet< GridPartType, SpaceType :: polynomialOrder > LagrangePointSetType;

      int errors = 0;
      for(const auto& entity : space)
      {
        const BasisFunctionSetType& basisSet = space.basisFunctionSet( entity );
        const int numBasisFct = basisSet.size();

        LagrangePointSetType pointSet( entity.type(), space.order( entity ) );
        const int numPoints = pointSet.size();

        std::vector< RangeType > phi( numPoints, RangeType( 0 ) );

        for( int i = 0; i < numPoints; ++i )
        {

          const DomainType& x = pointSet.point( i );

          // evaluate all basisFunctions on lagrange point i
          basisSet.evaluateAll( x, phi );

          if( std :: abs( phi[ i ][ 0 ] - 1.0 ) >= 1e-10 )
          {
            std :: cout << "Basis function " << i << " failed at " << x
                        << " (" << phi[ i ][ 0 ] << " != 1)!" << std :: endl;
            ++errors;
          }

          for( int j = 0; j < numBasisFct; ++j )
          {
            if( i == j )
              continue;

            // evaluate on lagrange point
            if( std :: abs( phi[ j ][ 0 ] ) >= 1e-10 )
            {
              std :: cout << "Basis function " << j << " failed at " << x
                          << " (" << phi[ j ][ 0 ] << " != 0)!" << std :: endl;
              ++errors;
            }
          }
        }
      }

      assert( errors == 0 );
    }

  } // namespace Fem

} // namespace Dune
